@@ -33,7 +33,9 @@ struct CCDBG_Build_opt : CDBG_Build_opt {
 
     bool outputColors;
 
-    CCDBG_Build_opt() : outputColors(true) {}
+    size_t min_nb_colors_search; // Minimum number of colors from each query that must occur in the graph. 0 = parameter must not be used.
+
+    CCDBG_Build_opt() : outputColors(true), min_nb_colors_search(0) {}
 };
 
 template<typename U = void> using UnitigColorMap = UnitigMap<DataAccessor<U>, DataStorage<U>>;
@@ -365,14 +367,140 @@ class ColoredCDBG : public CompactedDBG<DataAccessor<Unitig_data_t>, DataStorage
         /** Get the number of colors in the graph.
         * @return the number of colors in the graph.
         */
-        inline size_t getNbColors() const { return this->getData()->getNbColors(); }
+        inline size_t getNbColors() const {
 
+            return this->getData()->getNbColors();
+        }
+
+        /**
+        * Query the graph for input file(s) records, requiring a minimum ratio of k-mers, and write the results
+        * to disk in TSV format. Output is a binary matrix (|queries|,|colors|) with row and column names.
+        * @param query_filenames is a vector of input query files. Each file can be in FASTA or FASTQ format. Each record
+        * in each file is a query by default unless parameter "files_as_queries" is true.
+        * @param out_filename_prefix is the prefix of the output filename to which results are written.
+        * @param min_ratio_kmers is the minimum ratio of found k-mers (0 < ratio_kmers <= 1) in each query to report the query
+        * as present.
+        * @param inexact_search is a boolean indicating to search for k-mers containing up to one mismatch or indel with respect
+        * to the query k-mers and count as a hit any such inexact k-mer found in the graph.
+        * @param files_as_queries is a boolean indicating whether all records from each input query file constitute one query.
+        * @param nb_threads is an integer indicating how many threads can be used during the querying.
+        * @param verbose is a boolean indicating whether information messages must be printed during the execution of the function.
+        * @return Boolean indicating whether the querying completed successfully.
+        */
+        bool searchMinRatioKmer(const vector<string>& query_filenames, const string& out_filename_prefix,
+                                const double min_ratio_kmers,
+                                const bool inexact_search = false, const bool files_as_queries = false,
+                                const size_t nb_threads = 1, const bool verbose = false) const;
+
+        /**
+        * Query the graph for input file(s) records, requiring a minimum ratio of k-mers having a minimum number
+        * of colors, and write the results to disk in TSV format. Output is a binary matrix (|queries|,1) with row names.
+        * @param query_filenames is a vector of input query files. Each file can be in FASTA or FASTQ format. Each record
+        * in each file is a query by default unless parameter "files_as_queries" is true.
+        * @param out_filename_prefix is the prefix of the output filename to which results are written.
+        * @param min_ratio_kmers is the minimum ratio of found k-mers (0 < ratio_kmers <= 1) in each query to report the query
+        * as present.
+        * @param min_nb_colors is the minimum number of colors shared by found k-mers in each query to report that query as present.
+        * @param inexact_search is a boolean indicating to search for k-mers containing up to one mismatch or indel with respect
+        * to the query k-mers and count as a hit any such inexact k-mer found in the graph.
+        * @param files_as_queries is a boolean indicating whether all records from each input query file are one single query.
+        * @param nb_threads is an integer indicating how many threads can be used during the querying.
+        * @param verbose is a boolean indicating whether information messages must be printed during the execution of the function.
+        * @return Boolean indicating whether the querying completed successfully.
+        */
+        bool searchMinRatioKmer(const vector<string>& query_filenames, const string& out_filename_prefix,
+                                const double min_ratio_kmers, const size_t min_nb_colors,
+                                const bool inexact_search = false, const bool files_as_queries = false,
+                                const size_t nb_threads = 1, const bool verbose = false) const;
+
+        /**
+        * Query the graph for input file(s) records and write the number of found k-mers per color to disk
+        * in TSV format. Output is a uint or float matrix (|queries|,|colors|) with row and column names.
+        * @param query_filenames is a vector of input query files. Each file can be in FASTA or FASTQ format. Each record
+        * in each file is a query by default unless parameter "files_as_queries" is true.
+        * @param out_filename_prefix is the prefix of the output filename to which results are written.
+        * @param found_km_ratio_out is a boolean indicating to output the ratio of found k-mers from each query rather than
+        * the number of found k-mers.
+        * @param inexact_search is a boolean indicating to search for k-mers containing up to one mismatch or indel with respect
+        * to the query k-mers and count as a hit any such inexact k-mer found in the graph.
+        * @param files_as_queries is a boolean indicating whether all records from each input query file are one single query.
+        * @param nb_threads is an integer indicating how many threads can be used during the querying.
+        * @param verbose is a boolean indicating whether information messages must be printed during the execution of the function.
+        * @return Boolean indicating whether the querying completed successfully.
+        */
         bool search(const vector<string>& query_filenames, const string& out_filename_prefix,
-                    const double ratio_kmers, const bool get_nb_found_km, const bool get_ratio_found_km,
-                    const bool inexact_search, const size_t nb_threads, const bool verbose = false) const;
+                    const bool found_km_ratio_out = false, const bool inexact_search = false,
+                    const bool files_as_queries = false, const size_t nb_threads = 1, const bool verbose = false) const;
+
+        /**
+        * Query the graph for input file(s) records, requiring a minimum ratio of k-mers, and write the results
+        * to an opened output stream. Output is a binary matrix (|queries|,|colors|) with row and column names.
+        * @param query_filenames is a vector of input query files. Each file can be in FASTA or FASTQ format. Each record
+        * in each file is a query by default unless parameter "files_as_queries" is true.
+        * @param out is an output stream to which results are written. It must be opened prior to this function call and
+        * it is not closed by this function.
+        * @param min_ratio_kmers is the minimum ratio of found k-mers (0 < ratio_kmers <= 1) in each query to report the query
+        * as present.
+        * @param inexact_search is a boolean indicating to search for k-mers containing up to one mismatch or indel with respect
+        * to the query k-mers and count as a hit any such inexact k-mer found in the graph.
+        * @param files_as_queries is a boolean indicating whether all records from each input query file are one single query.
+        * @param nb_threads is an integer indicating how many threads can be used during the querying.
+        * @param verbose is a boolean indicating whether information messages must be printed during the execution of the function.
+        * @return Boolean indicating whether the querying completed successfully.
+        */
+        bool searchMinRatioKmer(const vector<string>& query_filenames, ostream& out, const double ratio_kmers,
+                                const bool inexact_search = false, const bool files_as_queries = false,
+                                const size_t nb_threads = 1, const bool verbose = false) const;
+
+        /**
+        * Query the graph for input file(s) records, requiring a minimum ratio of k-mers present in a minimum number
+        * of colors, and write the results to an opened output stream. Output is a binary matrix (|queries|,1) with row names.
+        * @param query_filenames is a vector of input query files. Each file can be in FASTA or FASTQ format. Each record
+        * in each file is a query by default unless parameter "files_as_queries" is true.
+        * @param out is an output stream to which results are written. It must be opened prior to this function call and
+        * it is not closed by this function.
+        * @param min_ratio_kmers is the minimum ratio of found k-mers (0 < ratio_kmers <= 1) in each query to report the query
+        * as present.
+        * @param min_nb_colors is the minimum number of colors shared by found k-mers in each query to report that query as present.
+        * @param inexact_search is a boolean indicating to search for k-mers containing up to one mismatch or indel with respect
+        * to the query k-mers and count as a hit any such inexact k-mer found in the graph.
+        * @param files_as_queries is a boolean indicating whether all records from each input query file are one single query.
+        * @param nb_threads is an integer indicating how many threads can be used during the querying.
+        * @param verbose is a boolean indicating whether information messages must be printed during the execution of the function.
+        * @return Boolean indicating whether the querying completed successfully.
+        */
+        bool searchMinRatioKmer(const vector<string>& query_filenames, ostream& out,
+                                const double min_ratio_kmers, const size_t min_nb_colors,
+                                const bool inexact_search = false, const bool files_as_queries = false,
+                                const size_t nb_threads = 1, const bool verbose = false) const;
+
+        /**
+        * Query the graph for input file(s) records and write the number of found k-mers per color to an opened output
+        * stream. Output is a uint/float matrix (|queries|,|colors|) with row and column names.
+        * @param query_filenames is a vector of input query files. Each file can be in FASTA or FASTQ format. Each record
+        * in each file is a query by default unless parameter "files_as_queries" is true.
+        * @param out is an opened output stream to which results are written. It must be opened prior to this function call and
+        * it is not closed by this function.
+        * @param found_km_ratio_out is a boolean indicating to output the ratio of found k-mers from each query rather than
+        * the number of found k-mers.
+        * @param inexact_search is a boolean indicating to search for k-mers containing up to one mismatch or indel with respect
+        * to the query k-mers and count as a hit any such inexact k-mer found in the graph.
+        * @param files_as_queries is a boolean indicating whether all records from each input query file are one single query.
+        * @param nb_threads is an integer indicating how many threads can be used during the querying.
+        * @param verbose is a boolean indicating whether information messages must be printed during the execution of the function.
+        * @return Boolean indicating whether the querying completed successfully.
+        */
+        bool search(const vector<string>& query_filenames, ostream& out,
+                    const bool found_km_ratio_out = false, const bool inexact_search = false,
+                    const bool files_as_queries = false, const size_t nb_threads = 1, const bool verbose = false) const;
 
     private:
 
+        bool searchMinRatioKmer_(   const vector<string>& query_filenames, ostream& out,
+                                    const double ratio_kmers, const size_t min_nb_colors,
+                                    const bool inexact_search, const bool files_as_queries,
+                                    const size_t nb_threads, const bool verbose) const;
+
         void checkColors(const vector<string>& filename_seq_in) const;
         bool loadColors(const string& input_graph_fn, const string& input_colors_fn, const size_t nb_threads, const bool verbose);
 
