@@ -15,7 +15,7 @@ MinimizerIndex::MinimizerIndex(const size_t sz, const double ratio_occupancy) :
 
         const size_t sz_with_empty = static_cast<size_t>((1.0 + (1.0 - ratio_occupancy)) * sz);
 
-        init_tables(sz_with_empty);
+        init_tables(max(sz_with_empty, static_cast<size_t>(BIFROST_MI_INIT_SZ)));
     }
 }
 
@@ -317,9 +317,9 @@ pair<MinimizerIndex::iterator, bool> MinimizerIndex::insert(const Minimizer& key
     if (size_ == 0) init_tables(BIFROST_MI_INIT_SZ);
     else if (pop >= static_cast<size_t>(size_ * max_ratio_occupancy)) {
 
-        size_t resize = 1.2 * size_;
+        size_t resize = max(1.2 * size_, static_cast<double>(1 + size_));
 
-        while (pop >= static_cast<size_t>(resize * max_ratio_occupancy)) resize *= 1.2;
+        while (pop >= static_cast<size_t>(resize * max_ratio_occupancy)) resize = max(1.2 * resize, static_cast<double>(1 + resize));
 
         reserve(resize);
     }
